@@ -6,8 +6,8 @@ class Ledger(HWIDevice):
     name = "Ledger"
     icon = "ledger_icon.svg"
 
-    def __init__(self, name, alias, keys, fullpath, manager):
-        HWIDevice.__init__(self, name, alias, keys, fullpath, manager)
+    def __init__(self, name, alias, keys, blinding_key, fullpath, manager):
+        HWIDevice.__init__(self, name, alias, keys, blinding_key, fullpath, manager)
 
     def create_psbts(self, base64_psbt, wallet):
         return {"hwi": wallet.fill_psbt(base64_psbt, non_witness=False)}
