@@ -4,6 +4,8 @@ from .hwi.specter_diy import enumerate as specter_enumerate, SpecterClient
 from ..helpers import to_ascii20
 from embit import bip32
 from embit.psbt import PSBT
+from embit.liquid.pset import PSET
+from embit.liquid.transaction import LSIGHASH
 from binascii import a2b_base64, b2a_base64
 
 
@@ -18,15 +20,31 @@ class Specter(SDCardDevice):
     qr_code_support_verify = True
     wallet_export_type = "qr"
     supports_hwi_multisig_display_address = True
+    liquid_support = True
 
-    def __init__(self, name, alias, keys, fullpath, manager):
-        super().__init__(name, alias, keys, fullpath, manager)
+    def __init__(self, name, alias, keys, blinding_key, fullpath, manager):
+        super().__init__(name, alias, keys, blinding_key, fullpath, manager)
 
     def create_psbts(self, base64_psbt, wallet):
+        try:
+            # remove rangeproofs and add sighash alls
+            psbt = PSET.from_string(base64_psbt)
+            for out in psbt.outputs:
+                out.range_proof = None
+                out.surjection_proof = None
+            for inp in psbt.inputs:
+                if not inp.sighash_type:
+                    inp.sighash_type = LSIGHASH.ALL
+            base64_psbt = psbt.to_string()
+        except:
+            pass
         psbts = super().create_psbts(base64_psbt, wallet)
         # remove non-witness utxo if they are there to reduce QR code size
         updated_psbt = wallet.fill_psbt(base64_psbt, non_witness=False, xpubs=False)
-        qr_psbt = PSBT.parse(a2b_base64(updated_psbt))
+        try:
+            qr_psbt = PSBT.from_string(updated_psbt)
+        except:
+            qr_psbt = PSET.from_string(updated_psbt)
         # find my key
         fgp = None
         derivation = None
@@ -39,7 +57,7 @@ class Specter(SDCardDevice):
         for inp in qr_psbt.inputs + qr_psbt.outputs:
             # keep only my derivation
             for k in list(inp.bip32_derivations.keys()):
-                if inp.bip32_derivations[k].fingerprint != fgp:
+                if fgp and inp.bip32_derivations[k].fingerprint != fgp:
                     inp.bip32_derivations.pop(k, None)
         # remove scripts from outputs (DIY should know about the wallet)
         for out in qr_psbt.outputs:
@@ -48,11 +66,10 @@ class Specter(SDCardDevice):
         # remove partial sigs from inputs
         for inp in qr_psbt.inputs:
             inp.partial_sigs = {}
-        psbts["qrcode"] = b2a_base64(qr_psbt.serialize()).strip().decode()
+        psbts["qrcode"] = qr_psbt.to_string()
+
         # we can add xpubs to SD card, but non_witness can be too large for MCU
-        psbts["sdcard"] = wallet.fill_psbt(
-            psbts["qrcode"], non_witness=False, xpubs=True
-        )
+        psbts["sdcard"] = wallet.fill_psbt(base64_psbt, non_witness=False, xpubs=True)
         return psbts
 
     def export_wallet(self, wallet):
