@@ -11,8 +11,11 @@ import subprocess
 import sys
 from collections import OrderedDict
 from mnemonic import Mnemonic
-from hwilib.tx import CTransaction
-from hwilib.psbt import PSBT
+from embit.script import Script
+from embit.psbt import PSBT
+from embit.transaction import Transaction
+from embit.liquid.pset import PSET
+from embit.liquid.transaction import LTransaction
 from .persistence import read_json_file, write_json_file
 from .util.bcur import bcur_decode
 import threading
@@ -26,7 +29,11 @@ defaultlock = threading.Lock()
 
 
 def is_testnet(chain):
-    return chain in ["test", "regtest", "signet"]
+    return chain not in ["main", "liquidv1", "None", "none", None, ""]
+
+
+def is_liquid(chain):
+    return chain not in ["main", "regtest", "test", "signet", "None", "none", None, ""]
 
 
 def locked(customlock=defaultlock):
@@ -164,7 +171,7 @@ def der_to_bytes(derivation):
 def get_devices_with_keys_by_type(app, cosigners, wallet_type):
     devices = []
     prefix = "tpub"
-    if app.specter.chain == "main":
+    if not is_testnet(app.specter.chain):
         prefix = "xpub"
     for cosigner in cosigners:
         device = copy.deepcopy(cosigner)
@@ -187,12 +194,14 @@ def get_devices_with_keys_by_type(app, cosigners, wallet_type):
 
 
 def clean_psbt(b64psbt):
-    psbt = PSBT()
-    psbt.deserialize(b64psbt)
+    try:
+        psbt = PSBT.from_string(b64psbt)
+    except:
+        psbt = PSET.from_string(b64psbt)
     for inp in psbt.inputs:
         if inp.witness_utxo is not None and inp.non_witness_utxo is not None:
             inp.non_witness_utxo = None
-    return psbt.serialize()
+    return psbt.to_string()
 
 
 def bcur2base64(encoded):
@@ -201,13 +210,13 @@ def bcur2base64(encoded):
 
 
 def get_txid(tx):
-    b = BytesIO(bytes.fromhex(tx))
-    t = CTransaction()
-    t.deserialize(b)
+    try:
+        t = Transaction.from_string(tx)
+    except:
+        t = LTransaction.from_string(tx)
     for inp in t.vin:
-        inp.scriptSig = b""
-    t.rehash()
-    return t.hash
+        inp.scriptSig = Script(b"")
+    return t.txid().hex()
 
 
 def get_startblock_by_chain(specter):
@@ -318,3 +327,13 @@ def is_ip_private(ip):
         or priv_16.match(ip)
     )
     return res is not None
+
+
+def get_address_from_dict(data_dict):
+    # TODO: Remove this helper function in favor of simple ["address"]
+    # when support for Bitcoin Core version < 22 is dropped
+    return (
+        data_dict["addresses"][0]
+        if "addresses" in data_dict and data_dict["addresses"][0]
+        else data_dict["address"]
+    )
