@@ -1,4 +1,4 @@
-import copy, random, json
+import copy, random, json, re
 
 from flask import (
     Flask,
@@ -15,9 +15,9 @@ from flask import current_app as app
 from mnemonic import Mnemonic
 from ..helpers import is_testnet, generate_mnemonic
 from ..key import Key
-from ..device_manager import get_device_class
+from ..managers.device_manager import get_device_class
 from ..devices.bitcoin_core import BitcoinCore
-from ..wallet_manager import purposes
+from ..managers.wallet_manager import purposes
 from ..specter_error import handle_exception
 
 rand = random.randint(0, 1e32)  # to force style refresh
@@ -38,7 +38,7 @@ def new_device_type():
     )
 
 
-@devices_endpoint.route("/new_device_keys/<device_type>", methods=["GET", "POST"])
+@devices_endpoint.route("/new_device_keys/<device_type>/", methods=["GET", "POST"])
 @login_required
 def new_device_keys(device_type):
     err = None
@@ -83,7 +83,7 @@ def new_device_keys(device_type):
                     err = "Failed to parse these xpubs:\n" + "\n".join(xpub)
                     break
         if not keys and not err:
-            if device_type == "bitcoincore":
+            if device_type in ["bitcoincore", "elementscore"]:
                 if not paths:
                     err = "No paths were specified, please provide at lease one."
                 if err is None:
@@ -147,11 +147,21 @@ def new_device_keys(device_type):
             device = app.specter.device_manager.add_device(
                 name=device_name, device_type=device_type, keys=keys
             )
-            flash("{} was added successfully!".format(device_name))
-            return redirect(
-                url_for("devices_endpoint.device", device_alias=device.alias)
-                + "?newdevice=true"
-            )
+            if app.specter.is_liquid:
+                return render_template(
+                    "device/device_blinding_key.jinja",
+                    new_device=True,
+                    device=device,
+                    error=err,
+                    specter=app.specter,
+                    rand=rand,
+                )
+            else:
+                flash("{} was added successfully!".format(device_name))
+                return redirect(
+                    url_for("devices_endpoint.device", device_alias=device.alias)
+                    + "?newdevice=true"
+                )
 
     return render_template(
         "device/new_device/new_device_keys.jinja",
@@ -170,9 +180,9 @@ def new_device_keys(device_type):
     )
 
 
-@devices_endpoint.route("/new_device_mnemonic/", methods=["GET", "POST"])
+@devices_endpoint.route("/new_device_mnemonic/<device_type>/", methods=["GET", "POST"])
 @login_required
-def new_device_mnemonic():
+def new_device_mnemonic(device_type):
     err = None
     strength = 128
     mnemonic = generate_mnemonic(strength=strength)
@@ -191,19 +201,17 @@ def new_device_mnemonic():
         passphrase = request.form["passphrase"]
         file_password = request.form["file_password"]
         existing_device = request.form.get("existing_device", None)
-        print("file_password")
-        print(file_password)
         if existing_device:
             existing_device = app.specter.device_manager.get_by_alias(existing_device)
         if not err:
             return render_template(
                 "device/new_device/new_device_keys.jinja",
+                device_class=get_device_class(device_type),
                 mnemonic=mnemonic,
                 passphrase=passphrase,
                 file_password=file_password,
                 range_start=range_start,
                 range_end=range_end,
-                device_class=BitcoinCore,
                 existing_device=existing_device,
                 error=err,
                 specter=app.specter,
@@ -212,6 +220,7 @@ def new_device_mnemonic():
 
     return render_template(
         "device/new_device/new_device_mnemonic.jinja",
+        device_type=device_type,
         strength=strength,
         mnemonic=mnemonic,
         existing_device=existing_device,
@@ -221,6 +230,38 @@ def new_device_mnemonic():
     )
 
 
+@devices_endpoint.route("/device_blinding_key/<device_alias>/", methods=["GET", "POST"])
+@login_required
+def device_blinding_key(device_alias):
+    err = None
+    try:
+        device = app.specter.device_manager.get_by_alias(device_alias)
+    except:
+        return render_template(
+            "base.jinja", error="Device not found", specter=app.specter, rand=rand
+        )
+    if not device:
+        return redirect(url_for("index"))
+    if request.method == "POST":
+        new_device = request.form.get("new_device", False)
+        blinding_key = request.form.get("blinding_key")
+        device.set_blinding_key(blinding_key)
+        if not new_device:
+            flash("Master blinding key was added successfully")
+        return redirect(
+            url_for("devices_endpoint.device", device_alias=device.alias)
+            + ("?newdevice=true" if new_device else "")
+        )
+
+    return render_template(
+        "device/device_blinding_key.jinja",
+        device=device,
+        error=err,
+        specter=app.specter,
+        rand=rand,
+    )
+
+
 # New device "manual" (deprecated)
 @devices_endpoint.route("/new_device_manual/", methods=["GET", "POST"])
 @login_required
@@ -395,9 +436,22 @@ def device(device_alias):
             device_type = request.form["device_type"]
             device.set_type(device_type)
     device = copy.deepcopy(device)
-    device.keys.sort(
-        key=lambda k: k.metadata["chain"] + k.metadata["purpose"], reverse=True
-    )
+
+    def sort_accounts(k):
+        # Ordering: 1) chain 2) account 3) purpose
+        pattern = r"^m\/([0-9]+)h\/([0-9])h\/([0-9]+)h"
+        if not k.derivation:
+            return 0
+        match = re.search(pattern, k.derivation)
+        if not match:
+            return 0
+        return (
+            int(match.group(1))
+            + (int(match.group(2)) + 1) * 1000000
+            + (int(match.group(3)) + 1) * 2000
+        )
+
+    device.keys.sort(key=sort_accounts, reverse=False)
     return render_template(
         "device/device.jinja",
         device=device,
