@@ -2,100 +2,59 @@ import copy
 import json
 import logging
 import os
-import traceback
-import random
-import time
-import zipfile
 import platform
+import random
 import secrets
-import requests
 import signal
-from io import BytesIO
-from .helpers import deep_update, clean_psbt, is_testnet
-from .util.checker import Checker
-from .rpc import autodetect_rpc_confs, detect_rpc_confs, get_default_datadir, RpcError
-from .bitcoind import BitcoindPlainController
-from .tor_daemon import TorDaemonController
-from urllib3.exceptions import NewConnectionError
-from requests.exceptions import ConnectionError
-from .rpc import BitcoinRPC
-from .device_manager import DeviceManager
-from .wallet_manager import WalletManager
-from .user_manager import UserManager
-from .persistence import write_json_file, read_json_file
-from .user import User
-from .util.price_providers import update_price
-from .util.tor import get_tor_daemon_suffix
 import threading
-from urllib.parse import urlparse
-from stem.control import Controller
-from .specter_error import SpecterError, ExtProcTimeoutException
+import time
+import traceback
+import zipfile
+from io import BytesIO
 from sys import exit
-from .util.setup_states import SETUP_STATES
-from .managers.otp_manager import OtpManager
+from urllib.parse import urlparse
+
+import requests
+from requests.exceptions import ConnectionError
+from stem.control import Controller
+from urllib3.exceptions import NewConnectionError
+
+from .helpers import clean_psbt, deep_update, is_liquid, is_testnet
+from .internal_node import InternalNode
+from .liquid.rpc import LiquidRPC
 from .managers.config_manager import ConfigManager
+from .managers.node_manager import NodeManager
+from .managers.otp_manager import OtpManager
+from .managers.user_manager import UserManager
+from .managers.wallet_manager import WalletManager
+from .node import Node
+from .persistence import read_json_file, write_json_file, write_node
+from .process_controller.bitcoind_controller import BitcoindPlainController
+from .rpc import (
+    BitcoinRPC,
+    RpcError,
+    autodetect_rpc_confs,
+    detect_rpc_confs,
+    get_default_datadir,
+)
+from .specter_error import ExtProcTimeoutException, SpecterError
+from .tor_daemon import TorDaemonController
+from .user import User
+from .util.checker import Checker
+from .util.price_providers import update_price
+from .util.setup_states import SETUP_STATES
+from .util.tor import get_tor_daemon_suffix
 
 logger = logging.getLogger(__name__)
 
 
-def get_rpc(
-    conf,
-    old_rpc=None,
-    return_broken_instead_none=False,
-    proxy_url="socks5h://localhost:9050",
-    only_tor=False,
-):
-    """
-    Checks if config have changed, compares with old rpc
-    and returns new one if necessary
-    If there is no working rpc-connection, it has to return None
-    If return_broken_instead_none is True, it'll return even a broken connection.
-    """
-    if "autodetect" not in conf:
-        conf["autodetect"] = True
-    rpc = None
-    if conf["autodetect"]:
-        if "port" in conf:
-            rpc_conf_arr = autodetect_rpc_confs(
-                datadir=os.path.expanduser(conf["datadir"]), port=conf["port"]
-            )
-        else:
-            rpc_conf_arr = autodetect_rpc_confs(
-                datadir=os.path.expanduser(conf["datadir"])
-            )
-        if len(rpc_conf_arr) > 0:
-            rpc = BitcoinRPC(**rpc_conf_arr[0], proxy_url=proxy_url, only_tor=only_tor)
-    else:
-        # if autodetect is disabled and port is not defined
-        # we use default port 8332
-        if not conf.get("port", None):
-            conf["port"] = 8332
-        rpc = BitcoinRPC(**conf)
-    if return_broken_instead_none:
-        return rpc
-    # check if we have something to compare with
-    if old_rpc is None:
-        return rpc if rpc and rpc.test_connection() else None
-    # check if we have something detected
-    if rpc is None:
-        # check if old rpc is still valid
-        return old_rpc if old_rpc.test_connection() else None
-    # check if something has changed and return new rpc if so.
-    # RPC cookie will have a new password if bitcoind is restarted.
-    if rpc.url == old_rpc.url and rpc.password == old_rpc.password:
-        return old_rpc
-    else:
-        logger.info("rpc config have changed.")
-        return rpc
-
-
 class Specter:
     """A central Object mostly holding app-settings"""
 
     # use this lock for all fs operations
     lock = threading.Lock()
 
-    def __init__(self, data_folder="./data", config={}):
+    def __init__(self, data_folder="./data", config={}, internal_bitcoind_version=""):
         if data_folder.startswith("~"):
             data_folder = os.path.expanduser(data_folder)
         data_folder = os.path.abspath(data_folder)
@@ -106,33 +65,30 @@ class Specter:
 
         self.data_folder = data_folder
 
-        # the rpc-object. Currently we only have one. If we have Node-Managers, we would need
-        # either many of them and register them with a keyword or something like that
-        self.rpc = None
-
-        # wallet that is currently rescanning with utxorescan
-        # can be only one at a time
-        self.utxorescanwallet = None
-
         self.user_manager = UserManager(self)
 
         self._config_manager = ConfigManager(self.data_folder, config)
 
+        self.internal_bitcoind_version = internal_bitcoind_version
+
+        # Migrating from Specter 1.3.1 and lower (prior to the node manager)
+        self.migrate_old_node_format()
+
+        self.node_manager = NodeManager(
+            proxy_url=self.proxy_url,
+            only_tor=self.only_tor,
+            active_node=self.active_node_alias,
+            bitcoind_path=self.bitcoind_path,
+            internal_bitcoind_version=internal_bitcoind_version,
+            data_folder=os.path.join(self.data_folder, "nodes"),
+        )
+
         self.torbrowser_path = os.path.join(
             self.data_folder, f"tor-binaries/tor{get_tor_daemon_suffix()}"
         )
 
-        self.bitcoind_path = os.path.join(
-            self.data_folder, "bitcoin-binaries/bin/bitcoind"
-        )
-
-        if platform.system() == "Windows":
-            self.bitcoind_path += ".exe"
-
-        self._bitcoind = None
         self._tor_daemon = None
 
-        self.node_status = None
         self.setup_status = {
             "stage": "start",
             "bitcoind": {
@@ -150,46 +106,12 @@ class Specter:
         # also loads and checks wallets for all users
         try:
             self.check(check_all=True)
-            rpc_conf = next(
-                (
-                    conf
-                    for conf in detect_rpc_confs(
-                        datadir=os.path.expanduser(
-                            self.config["rpc"]["datadir"]
-                            if self.config["rpc"].get("external_node", True)
-                            else self.config["internal_node"]["datadir"]
-                        )
-                    )
-                    if conf["port"] == 8332
-                ),
-                None,
-            )
 
             if os.path.isfile(self.torbrowser_path):
                 self.tor_daemon.start_tor_daemon()
         except Exception as e:
             logger.error(e)
 
-        if not self.config_manager.data["rpc"].get("external_node", True):
-            try:
-                self.bitcoind.start_bitcoind(
-                    datadir=os.path.expanduser(self.config["internal_node"]["datadir"]),
-                    timeout=15,  # At the initial startup, we don't wait on bitcoind
-                )
-            except ExtProcTimeoutException as e:
-                logger.error(e)
-                e.check_logfile(
-                    os.path.join(self.config["internal_node"]["datadir"], "debug.log")
-                )
-                logger.error(e.get_logger_friendly())
-            except SpecterError as e:
-                logger.error(e)
-                # Likely files of bitcoind were not found. Maybe deleted by the user?
-            finally:
-                try:
-                    self.set_bitcoind_pid(self.bitcoind.bitcoind_proc.pid)
-                except Exception as e:
-                    logger.error(e)
         self.update_tor_controller()
         self.checker = Checker(lambda: self.check(check_all=True), desc="health")
         self.checker.start()
@@ -209,9 +131,9 @@ class Specter:
             logger.info("Specter exit cleanup: Stopping Tor daemon")
             self._tor_daemon.stop_tor_daemon()
 
-        if self._bitcoind:
-            logger.info("Specter exit cleanup: Stopping bitcoind")
-            self._bitcoind.stop_bitcoind()
+        for node in self.node_manager.nodes.values():
+            if not node.external_node:
+                node.stop()
 
         logger.info("Closing Specter after cleanup")
         # For some reason we need to explicitely exit here. Otherwise it will hang
@@ -229,27 +151,16 @@ class Specter:
         # check if config file have changed
         self.check_config()
 
-        # update rpc if something doesn't work
-        rpc = self.rpc
-        if rpc is None or not rpc.test_connection():
-            rpc = get_rpc(
-                self.config_manager.rpc_conf,
-                self.rpc,
-                proxy_url=self.proxy_url,
-                only_tor=self.only_tor,
-            )
-
-        self.check_node_info()
+        self.node.update_rpc()
 
         # if rpc is not available
         # do checks more often, once in 20 seconds
-        if rpc is None or self.info.get("initialblockdownload", True):
+        if self.rpc is None or self.node.info.get("initialblockdownload", True):
             period = 20
         else:
             period = 600
         if hasattr(self, "checker") and self.checker.period != period:
             self.checker.period = period
-        self.rpc = rpc
 
         if not check_all:
             # find proper user
@@ -259,61 +170,29 @@ class Specter:
             for u in self.user_manager.users:
                 u.check()
 
+    @property
+    def node(self):
+        try:
+            return self.node_manager.active_node
+        except SpecterError as e:
+            self.update_active_node(list(self.node_manager.nodes.values())[0].alias)
+            return self.node_manager.active_node
+
+    @property
+    def rpc(self):
+        return self.node.rpc
+
+    @property
+    def utxorescanwallet(self):
+        return self.node.utxorescanwallet
+
     @property
     def config(self):
         """A convenience property simply redirecting to the config_manager"""
         return self.config_manager.data
 
-    def check_node_info(self):
-        self._is_configured = self.rpc is not None
-        self._is_running = False
-        if self._is_configured:
-            try:
-                res = [
-                    r["result"]
-                    for r in self.rpc.multi(
-                        [
-                            ("getblockchaininfo", None),
-                            ("getnetworkinfo", None),
-                            ("getmempoolinfo", None),
-                            ("uptime", None),
-                            ("getblockhash", 0),
-                            ("scantxoutset", "status", []),
-                        ]
-                    )
-                ]
-                self._info = res[0]
-                self._network_info = res[1]
-                self._info["mempool_info"] = res[2]
-                self._info["uptime"] = res[3]
-                try:
-                    self.rpc.getblockfilter(res[4])
-                    self._info["blockfilterindex"] = True
-                except:
-                    self._info["blockfilterindex"] = False
-                self._info["utxorescan"] = (
-                    res[5]["progress"]
-                    if res[5] is not None and "progress" in res[5]
-                    else None
-                )
-                if self._info["utxorescan"] is None:
-                    self.utxorescanwallet = None
-                self._is_running = True
-            except Exception as e:
-                self._info = {"chain": None}
-                self._network_info = {"subversion": "", "version": 999999}
-                logger.error("Exception %s while specter.check()" % e)
-                pass
-        else:
-            self._info = {"chain": None}
-            self._network_info = {"subversion": "", "version": 999999}
-
-        if not self._is_running:
-            self._info["chain"] = None
-
     def check_blockheight(self):
-        current_blockheight = self.rpc.getblockcount()
-        if self.info["blocks"] != current_blockheight:
+        if self.node.check_blockheight():
             self.check(check_all=True)
 
     def get_user_folder_id(self, user=None):
@@ -347,79 +226,7 @@ class Specter:
     # mark
     @property
     def bitcoin_datadir(self):
-        return self.config_manager.bitcoin_datadir
-
-    def abortrescanutxo(self):
-        self.rpc.scantxoutset("abort", [])
-        # Bitcoin Core doesn't catch up right away
-        # so app.specter.check() doesn't work
-        self._info["utxorescan"] = None
-        self.utxorescanwallet = None
-
-    def test_rpc(self, **kwargs):
-        conf = copy.deepcopy(self.config_manager.data["rpc"])
-        conf.update(kwargs)
-
-        rpc = get_rpc(
-            conf,
-            return_broken_instead_none=True,
-            proxy_url=self.proxy_url,
-            only_tor=self.only_tor,
-        )
-        if rpc is None:
-            return {"out": "", "err": "autodetect failed", "code": -1}
-        r = {}
-        r["tests"] = {"connectable": False}
-        r["err"] = ""
-        r["code"] = 0
-        try:
-            r["tests"]["recent_version"] = (
-                int(rpc.getnetworkinfo()["version"]) >= 170000
-            )
-            if not r["tests"]["recent_version"]:
-                r["err"] = "Core Node might be too old"
-
-            r["tests"]["connectable"] = True
-            r["tests"]["credentials"] = True
-            try:
-                rpc.listwallets()
-                r["tests"]["wallets"] = True
-            except RpcError as rpce:
-                logger.error(rpce)
-                r["tests"]["wallets"] = False
-                r["err"] = "Wallets disabled"
-
-            r["out"] = json.dumps(rpc.getblockchaininfo(), indent=4)
-        except ConnectionError as e:
-            logger.error("Caught an ConnectionError while test_rpc: %s", e)
-
-            r["tests"]["connectable"] = False
-            r["err"] = "Failed to connect!"
-            r["code"] = -1
-        except RpcError as rpce:
-            logger.error("Caught an RpcError while test_rpc: %s", rpce)
-            logger.error(rpce.status_code)
-            r["tests"]["connectable"] = True
-            r["code"] = rpc.r.status_code
-            if rpce.status_code == 401:
-                r["tests"]["credentials"] = False
-                r["err"] = "RPC authentication failed!"
-            else:
-                r["err"] = str(rpce.status_code)
-        except Exception as e:
-            logger.error(
-                "Caught an exception of type {} while test_rpc: {}".format(
-                    type(e), str(e)
-                )
-            )
-            r["out"] = ""
-            if rpc.r is not None and "error" in rpc.r:
-                r["err"] = rpc.r["error"]
-                r["code"] = rpc.r.status_code
-            else:
-                r["err"] = "Failed to connect"
-                r["code"] = -1
-        return r
+        return self.node.datadir
 
     # mark
     def _save(self):
@@ -430,23 +237,11 @@ class Specter:
         return os.path.join(self.data_folder, "config.json")
 
     # mark
-    def update_rpc(self, **kwargs):
-        need_update = self.config_manager.update_rpc(**kwargs)
-        if need_update:
-            self.rpc = get_rpc(
-                self.config_manager.rpc_conf,
-                None,
-                proxy_url=self.proxy_url,
-                only_tor=self.only_tor,
-            )
-            self._save()
-            self.check(check_all=True)
-        return self.rpc is not None
-
-    # mark
-    def set_bitcoind_pid(self, pid):
-        """set the control pid of the bitcoind daemon"""
-        self.config_manager.set_bitcoind_pid(pid)
+    def update_active_node(self, node_alias):
+        """update the current active node to use"""
+        self.config_manager.update_active_node(node_alias)
+        self.node_manager.switch_node(node_alias)
+        self.check()
 
     def update_setup_status(self, software_name, stage):
         self.setup_status[software_name]["error"] = ""
@@ -481,11 +276,6 @@ class Specter:
 
         return {"installed": installed, **self.setup_status[software_name]}
 
-    # mark
-    def update_use_external_node(self, use_external_node):
-        """set whatever specter should connect to internal or external node"""
-        self.config_manager.update_use_external_node(use_external_node)
-
     # mark
     def update_auth(self, method, rate_limit, registration_link_timeout):
         """simply persisting the current auth-choice"""
@@ -562,28 +352,9 @@ class Specter:
             "Tor daemon files missing. Make sure Tor is installed within Specter"
         )
 
-    @property
-    def bitcoind(self):
-        if os.path.isfile(self.bitcoind_path):
-            if not self._bitcoind:
-                self._bitcoind = BitcoindPlainController(
-                    bitcoind_path=self.bitcoind_path,
-                    rpcport=8332,
-                    network="mainnet",
-                    rpcuser=self.config["internal_node"]["user"],
-                    rpcpassword=self.config["internal_node"]["password"],
-                )
-            return self._bitcoind
-        raise SpecterError(
-            "Bitcoin Core files missing. Make sure Bitcoin Core is installed within Specter"
-        )
-
     def is_tor_dameon_running(self):
         return self._tor_daemon and self._tor_daemon.is_running()
 
-    def is_bitcoind_running(self):
-        return self._bitcoind and self._bitcoind.check_existing()
-
     @property
     def tor_controller(self):
         if self._tor_controller:
@@ -604,6 +375,10 @@ class Specter:
     def update_unit(self, unit, user):
         self.config_manager.update_unit(unit, user)
 
+    # mark
+    def update_hide_sensitive_info(self, hide_sensitive_info_bool, user):
+        self.config_manager.update_hide_sensitive_info(hide_sensitive_info_bool, user)
+
     # mark
     def update_price_check_setting(self, price_check_bool, user):
         self.config_manager.update_price_check_setting(price_check_bool, user)
@@ -649,41 +424,61 @@ class Specter:
         return self.rpc.estimatesmartfee(blocks)
 
     @property
-    def is_running(self):
-        return self._is_running
+    def bitcoind_path(self):
+        bitcoind_path = os.path.join(self.data_folder, "bitcoin-binaries/bin/bitcoind")
 
-    @property
-    def is_configured(self):
-        return self._is_configured
+        if platform.system() == "Windows":
+            bitcoind_path += ".exe"
+        return bitcoind_path
 
     @property
     def info(self):
-        return self._info
+        return self.node.info
 
     @property
     def network_info(self):
-        return self._network_info
+        return self.node.network_info
 
     @property
     def bitcoin_core_version(self):
-        return self.network_info["subversion"].replace("/", "").replace("Satoshi:", "")
+        return self.node.bitcoin_core_version
 
     @property
     def bitcoin_core_version_raw(self):
-        return self.network_info["version"]
+        return self.node.bitcoin_core_version_raw
 
     @property
     def chain(self):
-        return self._info["chain"]
+        return self.node.chain
+
+    @property
+    def network_parameters(self):
+        return self.node.network_parameters
 
     @property
     def is_testnet(self):
-        return is_testnet(self.chain)
+        return self.node.is_testnet
+
+    @property
+    def is_liquid(self):
+        return is_liquid(self.chain)
+
+    @property
+    def is_liquid(self):
+        return is_liquid(self.chain)
+
+    @property
+    def is_liquid(self):
+        return is_liquid(self.chain)
 
     @property
     def user_config(self):
         return self.config if self.user.is_admin else self.user.config
 
+    @property
+    def active_node_alias(self):
+        return self.user_config.get("active_node_alias", "default")
+
     @property
     def explorer(self):
         return self.user_config.get("explorers", {}).get(self.chain, "")
@@ -766,11 +561,24 @@ class Specter:
             self._otp_manager = OtpManager(self.data_folder)
         return self._otp_manager
 
+    @property
+    def hide_sensitive_info(self):
+        return self.user_config.get("hide_sensitive_info", False)
+
     def requests_session(self, force_tor=False):
         requests_session = requests.Session()
         if self.only_tor or force_tor:
-            requests_session.proxies["http"] = self.proxy_url
-            requests_session.proxies["https"] = self.proxy_url
+            proxy_url = self.proxy_url
+            proxy_parsed_url = urlparse(self.proxy_url)
+            proxy_url = proxy_parsed_url._replace(
+                netloc="{}:{}@{}".format(
+                    str(random.randint(10000, 0x7FFFFFFF)),
+                    "random",
+                    proxy_parsed_url.netloc,
+                )
+            ).geturl()
+            requests_session.proxies["http"] = proxy_url
+            requests_session.proxies["https"] = proxy_url
         return requests_session
 
     def specter_backup_file(self):
@@ -796,6 +604,63 @@ class Specter:
         memory_file.seek(0)
         return memory_file
 
+    # Migrating RPC nodes from Specter 1.3.1 and lower (prior to the node manager)
+    def migrate_old_node_format(self):
+        if not os.path.isdir(os.path.join(self.data_folder, "nodes")):
+            os.mkdir(os.path.join(self.data_folder, "nodes"))
+        old_rpc = self.config.get("rpc", None)
+        old_internal_rpc = self.config.get("internal_node", None)
+        if old_internal_rpc and os.path.isfile(self.bitcoind_path):
+            internal_node = InternalNode(
+                "Specter Bitcoin",
+                "specter_bitcoin",
+                old_internal_rpc.get("autodetect", False),
+                old_internal_rpc.get("datadir", get_default_datadir()),
+                old_internal_rpc.get("user", ""),
+                old_internal_rpc.get("password", ""),
+                old_internal_rpc.get("port", 8332),
+                old_internal_rpc.get("host", "localhost"),
+                old_internal_rpc.get("protocol", "http"),
+                os.path.join(
+                    os.path.join(self.data_folder, "nodes"), "specter_bitcoin.json"
+                ),
+                self,
+                self.bitcoind_path,
+                "mainnet",
+                "0.20.1",
+            )
+            write_node(
+                internal_node,
+                os.path.join(
+                    os.path.join(self.data_folder, "nodes"), "specter_bitcoin.json"
+                ),
+            )
+            del self.config["internal_node"]
+            if not old_rpc or not old_rpc.get("external_node", True):
+                self.config_manager.update_active_node("specter_bitcoin")
+
+        if old_rpc:
+            node = Node(
+                "Bitcoin Core",
+                "default",
+                old_rpc.get("autodetect", True),
+                old_rpc.get("datadir", get_default_datadir()),
+                old_rpc.get("user", ""),
+                old_rpc.get("password", ""),
+                old_rpc.get("port", None),
+                old_rpc.get("host", "localhost"),
+                old_rpc.get("protocol", "http"),
+                True,
+                os.path.join(os.path.join(self.data_folder, "nodes"), "default.json"),
+                self,
+            )
+            write_node(
+                node,
+                os.path.join(os.path.join(self.data_folder, "nodes"), "default.json"),
+            )
+            del self.config["rpc"]
+        self._save()
+
 
 class SpecterConfiguration:
     """An abstract class which only holds functionality relevant for storage of information mostly
